; ====================
; Medium level link include
; ====================

.include "CLAP/LowLevel.inc"

; Available routines:

;    = Writing
;        - sendByte

;    = Reading
;        - readByte

;    = Other
;        - sync

; These routines transfer at around 2.6 kbps,
; or over 300 bytes a second. It could be made
; faster, but this is fail safe and nicely modular,
; so I'll keep it at this speed for the moment :-)

linkDelay:
	ld b,25
linkDelay_loop:
	push hl
	pop hl
	djnz linkDelay_loop
	ret

; ============
; Sending a byte
; ============

; Send out the byte in a using a clocked link protocol

sendByte:
	ld d,128		; bitmask
	ld c,a		; save byte
	call setHighHigh	; Pull the lines up
	
sendByte_loop:
	ld a,c
	and d		; mask out one bit
	or a		; compare zero
	jp z,sendByte_zerobit
	call setRingLow	; Bit is one, pull ring low
	jp sendByte_toggle
sendByte_zerobit:
	call setRingHigh	; Bit is zero, set ring high
sendByte_toggle:
	call toggleTip	; Pull the clock low
	call linkDelay	; Wait
	call toggleTip	; Reset the clock
	call linkDelay	; Wait
	ld a,d
	rra		; Rotate bitmask right
	or a		; Are we done?
	ret z
	ld d,a
	jp sendByte_loop	; Next bit
	
; ============
; Reading a byte
; ============

; Receive a byte in a and c using a clocked link protocol
; Clear exits prematurely and will return 191 in a

readByteTimeOut:
	ld e,1
	jr readByte_go
readByte:
	ld e,0
readByte_go:
	ld d,128		; bitmask
	ld c,0		; byte
	ld b,$FF		; timeout
	call setHighHigh	; Don't interfere with the transmission
	
readByte_loop:
	ld a,e		; Are we waiting too long?
	or a
	call nz,readByte_timeout

	ld a,$FF		; Scan for Exit
	out (1),a
	ld a,$FD
	out (1),a
	in a,(1)
	cp 191
	ret z
	
	call readTip	; Wait for tip to go low
	jr nz,readByte_loop
	
	call readRing	; Read ring for data bit
	jp nz,readByte_zerobit
	ld a,c
	or d		; Write bit
	ld c,a
readByte_zerobit:		; Don't write bit
	ld a,d
	rrca		; Rotate bitmask right
	ld d,a
	
readByte_wait:		; Wait for tip to go high
	call readTip
	jr z,readByte_wait
	
	ld a,d
	cp 128
	ld a,c
	ret z		; Are we done?
	
	jp readByte_loop	; Next bit
	
readByte_timeout:
	dec b
	ret nz
	pop de
	ret

; ====================
; Synchronizing calculators
; ====================

; Synchronize two (N.B.: two, no more) calculators
; Identification number is returned in a (0 or 1)
; Clear exits prematurely and will return 191 in a

sync:
	call signal
	call linkDelay
	call readLines	; Check for a response
	cp 3
	ld a,1		; I'm calc B
	ret z		; We're synchronized
	
	; Otherwise...
	
sync_loop:
	ld a,$FF		; Scan for Exit
	out (1),a
	ld a,$FD
	out (1),a
	in a,(1)
	cp 191
	ret z
	
	call readLines	; Wait for a response
	cp 3
	jr nz,sync_loop
sync_loop2:
	call readlines	; Wait for release
	cp 3
	jr z,sync_loop2
	
	call signal
	ld a,0		; I'm calc A
	ret
	
signal:
	call setLowLow	; Pull the lines low to ask for Sync
	call linkDelay
	call setHighHigh	; And set 'em back
	ret

medlevelend:
.echo "Medium level link included, size is "
.echo medlevelend-linkDelay
.echo " bytes\n"
