; ====================
; Network level link include
; ====================

.include "CLAP/MidLevel.inc"

; Available routines:

;    = Initialization
;        - login
;        - getMyAddress
;
;    = Writing
;        - send
;
;    = Reading
;        - read

myAddress:
	.db 0
highestAddress:
	.db 0

; ===================
; Loging in on the network
; ===================

; Logs you in, meaning that you request an address from the network
; If no network is present and no address is received, the task of
; network master will be taken up.
; Returns success in z flag:
;   z     - succes
;   nz   - failure
; Returns my address in a when succesful

login:
	
	ld a,1		; Send address request
	call sendByte
	xor a
	call sendByte
	xor a
	call sendByte
	
	call readByteTimeOut
	ld c,a
	ld a,b
	or a
	jr z,im_master
	
	ld a,c		; First received byte should be 0
	or a
	ret nz
	
	call readByte	; Second received byte should be 1
	cp 1
	ret nz
	
	call readByte	; Third received byte should be 1
	cp 1
	ret nz
	
	call readByte	; Fourth received byte is my address
	ld (myAddress),a
	
	ret
	
im_master:
	
	ld a,1
	ld (myAddress),a
	ld (highestAddress),a
	ret
	
	
; Returns my network address in a

getMyAddress:

	ld a,(myAddress)
	ret

; =======================	
; Sending data to another calc
; =======================

; Sends data to a receipient in the right network format
; Input parameters:
; a  = Receipient
; b  = Data size
; hl = Pointer to data

send:

	push bc
	call sendByte	; Send receipient address
	
	ld a,(myAddress)
	call sendByte	; Send my address

	pop bc
	push bc
	ld a,b
	call sendByte	; Send data size

	pop bc
send_loop:
	push bc
	ld a,(hl)
	call sendByte	; Send data
	inc hl
	pop bc
	djnz send_loop
	ret		; Done
	
; ========================
; Reading data from the network
; ========================

; Waits for a message from the network that is addressed to this calc
; If a new calculators tries to join the network this routine will give it an
; address if this calculator is the master calc.
; Sent data is stored at hl
; Data length is returned in c
; Sender address is returned in d, will be 0 when a client
;    has been added instead of a message received

read:

	call readByte	; Wait for a transmission

	ld a,$FF		; Scan for Exit
	out (1),a
	ld a,$FD
	out (1),a
	in a,(1)
	cp 191
	ret z
	
	ld a,c
	cp $FF		; Is it addressed to everyone?
	jp z,for_me
	ld a,(myAddress)	; Is it addressed to me?
	cp c
	jp nz,not_for_me
	
for_me:
	call readByte	; Yes it is, who is it from?
	or a		; Is it an address Request?
	jp z,read_adrRequest
	ld d,a
	push de
	
	call readByte	; How long is the message?
	ld c,a
	or a
	jr z,read_done	; Size is zero
	ld b,a

recv_loop:	
	push bc

	call readByte	; Read a byte
	ld (hl),a		; And store it
	inc hl
	
	pop bc
	djnz recv_loop	; Untill the message is done
	
read_done:
	pop de		; d = sender, c = size
	ret
	
read_adrRequest:
	call readByte	; Dummy read on the size
	xor a
	call sendByte	; Address new calc
	ld a,(myAddress)
	call sendByte	; Give sender address
	ld a,1
	call sendByte	; We're sending a 1 byte message
	ld a,(highestAddress)
	inc a
	ld (highestAddress),a
	call sendByte	; This is your address
	ld d,0
	ret
	
not_for_me:
	call readByte	; Dummy read on the sender
	call readByte	; Read the size
	or a
	jp z,read
	ld b,a
not_for_me_loop:
	push bc
	call readByte	; <size> more dummy reads for the content
	pop bc
	djnz not_for_me_loop
	jp read		; And wait for another message


networklevelend:
.echo "Network level link included, size is "
.echo networklevelend-myAddress
.echo " bytes\n"
