; ===============================================================
; Required Headers
; ===============================================================

.include "Includes/headers.inc"

#define api_defines
#include "Includes/current_api.inc"

; ===============================================================
; Main entry point
; ===============================================================

numCalcs		=	4

#define	scrErase		100
#define	prgExit		200
#define	cpyBuffer		150

#define	scrWidth		numCalcs * 96 - 7
#define	scrHeight		64 - 6

#define	ball_x		savesscreen
#define	ball_y		ball_x+2
#define	ball_dx		ball_y+1
#define	ball_dy		ball_dx+2

main

	call login
	jr nz,main		; Try logging in untill succesful
	
	cp 1		; Am I the master calc?
	jp nz,slave
	
; =================
; Master program code
; =================
	
master:

	bcall(_clrLCDFull)
	bcall(_homeup)
	ld hl,waiting	; Display "Waiting for slaves"
	bcall(_puts)
	
.if numCalcs > 1
	
	ld b,numCalcs-1
master_loop:
	push bc
	call read		; Wait untill 3 slaves are added to the network
	pop bc
	djnz master_loop
	
.endif
	
	; At this point, I have numCalcs - 1 slaves at my control
	
start_bouncing:
	
	; Initialize bouncing ball
	
	ld hl,50
	ld (ball_x),hl
	ld a,25
	ld (ball_y),a
	ld hl,1
	ld (ball_dx),hl
	ld a,-1
	ld (ball_dy),a
	
ball_loop:
	
	call display	; Show the ball (anywhere on the screens)
	call updateCoords	; Move the ball
	call bigDelay	; Wait

	ld a,$FF		; Scan for Exit
	out (1),a
	ld a,$FD
	out (1),a
	in a,(1)
	cp 191
	jp nz,ball_loop	; And repeat
	
.if numCalcs == 4

	call showClapIsCool
	
wait_release:
	xor a	; ld a,0
	out (1),a
	in a,(1)
	cp $FF
	jr nz,wait_release
wait_keypress:
	xor a	; ld a,0
	out (1),a
	in a,(1)
	cp $FF
	jr z,wait_keypress
        
.endif
	
	ld hl,order
	ld a,prgExit	; Quit it y'all
	ld (hl),a
	ld a,$FF
	ld b,1
	call send
	
	ret
	
.if numCalcs == 4

showClapIsCool:

	ld a,scrErase	; Clear your screens y'all
	ld hl,order
	ld (hl),a
	ld b,1
	ld a,$FF
	call send
	
	call eraseScreen	; Clear my own screen too

	ld hl,coolTable	; Get my cool table!
	ld b,(hl)
	inc hl
	
clapIsCool_loop:
	ld a,(hl)		; Show all the balls
	ld d,a
	inc hl
	ld a,(hl)
	ld e,a
	ld (ball_x),de
	inc hl
	ld a,(hl)
	ld (ball_y),a
	inc hl
	push hl
	push bc
	call display_dontClear
	pop bc
	pop hl
	djnz clapIsCool_loop
	
	ret
	
coolTable:
	.db 79
	
	; C		7
	.db 0,10,10
	.db 0,18,10
	.db 0,2,20
	.db 0,2,30
	.db 0,2,40
	.db 0,10,50
	.db 0,18,50
	
	; L		7
	.db 0,35,10
	.db 0,35,20
	.db 0,35,30
	.db 0,35,40
	.db 0,35,50
	.db 0,43,50
	.db 0,51,50
	
	; A		10
	.db 0,68,20
	.db 0,68,30
	.db 0,68,40
	.db 0,68,50
	.db 0,76,10
	.db 0,76,30
	.db 0,84,20
	.db 0,84,30
	.db 0,84,40
	.db 0,84,50
	
	; P		8
	.db 0,101,10
	.db 0,101,20
	.db 0,101,30
	.db 0,101,40
	.db 0,101,50
	.db 0,109,10
	.db 0,109,30
	.db 0,118,20
	
	; I		9
	.db 0,167,10
	.db 0,167,50
	.db 0,174,10
	.db 0,174,20
	.db 0,174,30
	.db 0,174,40
	.db 0,174,50
	.db 0,181,10
	.db 0,181,50
	
	; S		7
	.db 0,198,20
	.db 0,198,50
	.db 0,205,10
	.db 0,205,30
	.db 0,205,50
	.db 0,212,10
	.db 0,212,40
	
	; C		7
	.db 1,13,10
	.db 1,21,10
	.db 1,5,20
	.db 1,5,30
	.db 1,5,40
	.db 1,13,50
	.db 1,21,50
	
	; O		8
	.db 1,38,20
	.db 1,38,30
	.db 1,38,40
	.db 1,46,10
	.db 1,46,50
	.db 1,54,20
	.db 1,54,30
	.db 1,54,40
	
	; O		8
	.db 1,71,20
	.db 1,71,30
	.db 1,71,40
	.db 1,79,10
	.db 1,79,50
	.db 1,87,20
	.db 1,87,30
	.db 1,87,40
	
	; L		7
	.db 1,104,10
	.db 1,104,20
	.db 1,104,30
	.db 1,104,40
	.db 1,104,50
	.db 1,112,50
	.db 1,120,50

.endif
	
; Move the ball and make it bounce
	
updateCoords:

	ld hl,(ball_dx)
	ex de,hl
	ld hl,(ball_x)	; Add dx to x
	add hl,de
	ld (ball_x),hl
	
	ld a,(ball_dy)
	ld b,a
	ld a,(ball_y)	; Add dy to y
	add a,b
	ld (ball_y),a
	
	; Check for collisions
	
	ld hl,(ball_x)		; Left/right collision
	ld a,h
	or l
	call z,bounce_left
	ld bc,scrWidth
	call cpHLwithBC
	call z,bounce_right
	
	ld a,(ball_y)		; Top/bottom collision
	or a
	call z,bounce_top
	cp scrHeight
	call z,bounce_bottom
	
	ret
	
bounce_left:
	ld hl,1
	ld (ball_dx),hl
	ret
bounce_right:
	ld hl,-1
	ld (ball_dx),hl
	ret
bounce_top:
	push af
	ld a,1
	ld (ball_dy),a
	pop af
	ret
bounce_bottom:
	ld a,-1
	ld (ball_dy),a
	ret
	
; Treats the four screens as one large display
	
display:

	ld a,scrErase	; Clear your screens y'all
	ld hl,order
	ld (hl),a
	ld b,1
	ld a,$FF
	call send
	
	call eraseScreen	; Clear my own screen too

display_dontClear:

	ld hl,(ball_x)
	ld bc,96-3
	call cpHLwithBC	; Is the ball on my screen?
	jp nc,right_half_1

	ld a,(ball_y)
	ld d,l
	ld e,a
	
	; First screen
	
	sprite.drawXor(sphere,de)
	
	call bigDelay
	jp order_copyBuffer
	
.if numCalcs == 3			; Workaround for bug in Brass
				; (for loop doesn't work for one iteration)
right_half_1:
	; hl = still ball_x
	ld bc,-3+(2*96)
	call cpHLwithBC		; Is the ball on my screen?
	jp nc,right_half_2		; If not, move to next screen

	ld bc,96
	sbc hl,bc			; If so, substract screens on the left
	inc l
	ld d,l
	
	ld a,(ball_y)
	ld e,a
	
	; Middle screen(s)
	
	ld c,2
	jp order_draw		; And draw

.elseif numCalcs > 3

.for i, 1, numCalcs-2

right_half_{i}:
	; hl = ball_x
	
	ld bc,-3+((i+1)*96)
	call cpHLwithBC		; Is the ball on my screen?
	jp nc,right_half_{i+1}	; If not, move to next screen

	ld bc,i*96
	sbc hl,bc			; If so, substract screens on the left
	inc l
	ld d,l
	
	ld a,(ball_y)
	ld e,a
	
	; Middle screen(s)
	
	ld c,i+1
	jp order_draw		; And draw
	
.loop

.endif
	
right_half_{numCalcs-1}:
	; hl = still ball_x
	
	ld bc,96*(numCalcs-1)	; I'm the last screen
	sbc hl,bc
	ld d,l
	
	ld a,(ball_y)
	ld e,a
	
	; Last screen
	
	ld c,numCalcs

order_draw:
	ld a,d			; x coord
	ld (order),a
	ld a,e			; y coord
	ld (order+1),a
	ld hl,order
	ld b,2
	ld a,c			; Slave id
	call send			; Send order
	
order_copyBuffer:
	ld a,cpyBuffer		; Draw on slave screens
	ld hl,order
	ld (hl),a
	ld b,1
	ld a,$FF			; All slaves
	call send
	
	jp ionFastCopy		; Draw on the master screen

; Delay for the boucing loop

bigDelay:
	ld b,200
bigDelay_loop:
	push hl
	inc hl
	dec hl
	pop hl
	djnz bigDelay_loop
	ret

; Double register compare
	
cpHLwithBC:
	ld a,h
	cp b
	ret nz
	ld a,l
	cp c
	ret
	
; ================
; Slave program code
; ================
	
slave:
wait_for_order:
	ld hl,order
	call read			; Wait for an order
	
	ld a,$FF			; Scan for Exit
	out (1),a
	ld a,$FD
	out (1),a
	in a,(1)
	cp 191
	ret z
	
	ld a,(order)
	cp prgExit			; If order is exit
	ret z			; Exit
	cp scrErase		; If order is clear screen
	jp z,clearScreen		; Clear the screen
	cp cpyBuffer		; If order is copy buffer
	jp z,copyBuffer		; You guessed it :)

	ld d,a			; Otherwise this is x
	ld a,(order+1)
	ld e,a			; And the next is y
	sprite.drawXor(sphere,de)	; Draw my sphere, you slave!
	jp wait_for_order
	
clearScreen:
	call eraseScreen
	jp wait_for_order		; And back to business
	
eraseScreen:
	ld hl,plotsscreen		; Clear the screen
	xor a
	ld (hl),a
	ld de,plotsscreen+1
	ld bc,767
	ldir
	ret

copyBuffer:
	call ionFastCopy
	jp wait_for_order
	
; ==========
; Data storage
; ==========

order:
	.db 0,0
waiting:
	.db "Waiting for     slaves...",0
sphere:
	.db 6
	.db %00011000
	.db %00110100
	.db %01111010
	.db %01111110
	.db %00111100
	.db %00011000
	
; =======
; Libraries
; =======

.include "CLAP/Network.inc"

#define api_routines
#include "Includes/current_api.inc"

.end

